use raylib::prelude::*;
use serde::{Deserialize, Serialize};

use crate::{
	input::Input,
	theme::FG_CHAPTER_TITLE,
	ui::{text_button, toggle_button},
	util::Scroll,
	Globals,
};

#[derive(Default, Debug, Clone, Serialize, Deserialize)]
pub struct Config {
	pub input: Input,
	#[serde(default)]
	pub show_debug_timing: bool,
	#[serde(default)]
	pub show_power_direction: bool,
	#[serde(skip)]
	scroll_offset: u32,
}

pub enum MenuReturn {
	Stay,
	StaySave,
	ReturnSave,
	ReturnCancel,
}

impl Config {
	#[must_use]
	pub fn draw_edit(&mut self, d: &mut RaylibDrawHandle, globals: &mut Globals) -> MenuReturn {
		match globals.mouse.scroll() {
			Some(Scroll::Down) => self.scroll_offset += 64,
			Some(Scroll::Up) => self.scroll_offset = self.scroll_offset.saturating_sub(64),
			None => (),
		}
		let mut y = -(self.scroll_offset as i32) + 15;
		d.draw_text("Settings", 16, y, 30, FG_CHAPTER_TITLE);
		y += 40;

		if text_button(d, &globals.mouse, 10, y, 80, "apply") {
			return MenuReturn::StaySave;
		}
		if text_button(d, &globals.mouse, 100, y, 80, "done") {
			return MenuReturn::ReturnSave;
		}
		if text_button(d, &globals.mouse, 190, y, 80, "cancel") {
			return MenuReturn::ReturnCancel;
		}
		y += 40;

		let mut toggle = |value, text| {
			toggle_button((d, &globals.mouse), 10, y, 30, 30, value);
			d.draw_text(text, 50, y + 5, 20, Color::WHITE);
			y += 40;
		};

		toggle(&mut self.show_power_direction, "show power directions");
		toggle(&mut self.show_debug_timing, "show debug timing");

		// self.input.update(d);
		self.input.draw_edit(d, globals, y);
		MenuReturn::Stay
	}
}
